package auth //nolint:testpackage

import (
	"bytes"
	"context"
	"testing"
	"time"

	"github.com/rs/zerolog"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"gopkg.in/dgrijalva/jwt-go.v3"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
	"orus.io/orus-io/rednerd/models"
)

func TestAuth(t *testing.T) {
	t.Run("cache full log message", func(t *testing.T) {
		t.Run("small cache", func(t *testing.T) {
			buf := bytes.NewBuffer(nil)
			log := zerolog.New(buf)
			ctx := log.WithContext(context.Background())
			testdb := database.GetTestDB(ctx, t, migration.Source)
			defer testdb.Close()

			auth := NewAuth(TokenOptions{
				Secret:          "test",
				Expiration:      time.Minute,
				CacheMaxSize:    10,
				CachePurgeDelay: time.Minute,
			}, testdb.DB, log)
			defer auth.Close()

			for _, id := range []string{
				"1", "2", "3",
			} {
				_, _, err := auth.MakeToken(models.Principal{
					StandardClaims: jwt.StandardClaims{
						Subject: id,
					},
				}, false)
				require.NoError(t, err)
			}

			time.Sleep(time.Second)

			for _, id := range []string{
				"1", "2", "3",
				"4", "5", "6",
				"7", "8", "9",
				"10", "11", "12",
			} {
				_, _, err := auth.MakeToken(models.Principal{
					StandardClaims: jwt.StandardClaims{
						Subject: id,
					},
				}, false)
				require.NoError(t, err)
			}

			assert.Equal(t, 10, auth.cache.ItemCount())
			assert.JSONEq(t, `{
				"level":"warn",
				"counters":{
					"total":10,
					"users":{
						"1":2,
						"2":2,
						"3":2,
						"4":1,
						"5":1,
						"6":1,
						"7":1
					}
				},
				"message":"auth token cache is full"
			}`, buf.String())
		})
	})
}
