package models

import (
	"errors"
	"fmt"
	"net/http"
	"strconv"
)

// FromError builds a Error from a error interface.
func FromError(err error) *Error {
	var errError *Error
	if errors.As(err, &errError) {
		return errError
	}
	msg := err.Error()

	return &Error{
		Message: msg,
	}
}

// NewError builds a Error.
func NewError(code int64, message string) *Error {
	return &Error{code, message}
}

// NewErrorf builds a Error from a format string and args.
func NewErrorf(code int64, format string, args ...interface{}) *Error {
	message := fmt.Sprintf(format, args...)

	return &Error{code, message}
}

func (m Error) Error() string {
	if m.Message != "" {
		return fmt.Sprintf("%d %s", m.Code, m.Message)
	}

	return strconv.Itoa(int(m.Code))
}

func (m Error) IsEmpty() bool {
	return m.Code == 0 && m.Message == ""
}

func Unauthorized(message string) *Error {
	return NewError(http.StatusUnauthorized, message)
}

func NotFound(message string) *Error {
	return NewError(http.StatusNotFound, message)
}

func BadRequest(message string) *Error {
	return NewError(http.StatusBadRequest, message)
}
