package models

import (
	"encoding/base64"
	"errors"
	"fmt"
)

// ErrBadBodyFormat is returned when the input body format is invalid or inconsistent.
var ErrBadBodyFormat = errors.New("bad body format")

// Bytes is the document content as a slice of bytes.
func (t *Template) Bytes() ([]byte, error) {
	switch t.BodyFormat {
	case "":
		fallthrough
	case "base64":
		b, err := base64.StdEncoding.DecodeString(t.Body)
		if err != nil {
			err = fmt.Errorf("%w: %w", ErrBadBodyFormat, err)
		}

		return b, err
	case "text":
		return []byte(t.Body), nil
	default:
		return []byte(t.Body), nil
	}
}

// String is the document content as a string.
func (t *Template) String() (string, error) {
	switch t.BodyFormat {
	case "":
		fallthrough
	case "base64":
		b, err := base64.StdEncoding.DecodeString(t.Body)
		if err != nil {
			return "", err
		}

		return string(b), nil
	case "text":
		return t.Body, nil
	default:
		return t.Body, nil
	}
}
