package rendering

import (
	"strings"

	"github.com/rs/zerolog"
)

//nolint:revive
type RenderingEngineSetup struct {
	Name           string
	GetOptionGroup func() interface{}
	Factory        func(optionGroup interface{}, log zerolog.Logger) (Engine, error)
}

var renderingEngineSetupRegistry []RenderingEngineSetup

func RegisterRenderingEngineSetup(
	name string,
	getOptionGroup func() interface{},
	factory func(optionGroup interface{}, log zerolog.Logger) (Engine, error),
) {
	for _, s := range renderingEngineSetupRegistry {
		if strings.EqualFold(s.Name, name) {
			panic("duplicate rendering engine setup: " + name)
		}
	}
	renderingEngineSetupRegistry = append(renderingEngineSetupRegistry, RenderingEngineSetup{
		Name:           name,
		GetOptionGroup: getOptionGroup,
		Factory:        factory,
	})
}

func ListRenderingEngineSetup() []RenderingEngineSetup {
	return renderingEngineSetupRegistry
}

type TemplateEngineSetup struct {
	Name           string
	GetOptionGroup func() interface{}
	Factory        func(optionGroup interface{}, log zerolog.Logger) (TemplateEngine, error)
}

var templateEngineSetupRegistry []TemplateEngineSetup

func RegisterTemplateEngineSetup(
	name string,
	getOptionGroup func() interface{},
	factory func(optionGroup interface{}, log zerolog.Logger) (TemplateEngine, error),
) {
	for _, s := range templateEngineSetupRegistry {
		if strings.EqualFold(s.Name, name) {
			panic("duplicate rendering engine setup: " + name)
		}
	}
	templateEngineSetupRegistry = append(templateEngineSetupRegistry, TemplateEngineSetup{
		Name:           name,
		GetOptionGroup: getOptionGroup,
		Factory:        factory,
	})
}

func ListTemplateEngineSetup() []TemplateEngineSetup {
	return templateEngineSetupRegistry
}
