package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/template_management"
)

// NewAccountTemplateDeleteHandler returns a AccountTemplateDeleteHandler.
func NewAccountTemplateDeleteHandler(db *sqlx.DB) *AccountTemplateDeleteHandler {
	return &AccountTemplateDeleteHandler{
		db: db,
	}
}

// AccountTemplateDeleteHandler handles GET /user.
type AccountTemplateDeleteHandler struct {
	db *sqlx.DB
}

func (h *AccountTemplateDeleteHandler) handle(
	ctx context.Context, account, name string,
) error {
	log := zerolog.Ctx(ctx)

	_, err := database.ExecContext(
		ctx, h.db,
		squirrel.
			Delete(models.TemplateDBTable).
			Where(squirrel.Eq{
				models.TemplateDBAccountColumn: account,
				models.TemplateDBNameColumn:    name,
			}),
		log,
	)

	return err
}

// Handle handles the incoming request.
func (h *AccountTemplateDeleteHandler) Handle(
	params template_management.AccountTemplateDeleteParams, principal *models.Principal,
) middleware.Responder {
	if principal.IsAdmin {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(IsAdminError)
	}
	if principal.Subject != params.Account {
		return template_management.
			NewAccountTemplateAddUnauthorized().
			WithPayload(IsDifferentUserError)
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	err := h.handle(ctx, params.Account, params.Name)
	if err != nil {
		return HandleError(err, log)
	}

	return template_management.NewAccountTemplateDeleteNoContent()
}
