package handlers

import (
	"net/http"

	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"

	"orus.io/orus-io/rednerd/auth"
	"orus.io/orus-io/rednerd/models"
	op_auth "orus.io/orus-io/rednerd/restapi/operations/auth"
)

// NewAuthRefreshTokenHandler ...
func NewAuthRefreshTokenHandler(db *sqlx.DB, auth *auth.Auth) *AuthRefreshToken {
	return &AuthRefreshToken{db, auth}
}

// AuthRefreshToken handles /auth/renew_token.
type AuthRefreshToken struct {
	db   *sqlx.DB
	auth *auth.Auth
}

func (h *AuthRefreshToken) handle(principal *models.Principal) (string, error) {
	// TODO Query the db and check it the profile is still valid (ie has an api key set),
	// is still connected to the account, and the account is still active

	token, _, err := h.auth.MakeToken(*principal, false)

	return token, err
}

// Handle handles the request.
func (h *AuthRefreshToken) Handle(
	params op_auth.RefreshTokenParams, principal *models.Principal,
) middleware.Responder {
	token, err := h.handle(principal)
	if err != nil {
		return op_auth.NewRefreshTokenDefault(http.StatusInternalServerError).WithPayload(models.FromError(err))
	}

	return op_auth.NewRefreshTokenOK().WithPayload(token)
}
