package handlers

import (
	"context"
	"testing"

	"github.com/stretchr/testify/require"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/migration"
	"orus.io/orus-io/rednerd/testutils"
)

func TestUserListHandler(t *testing.T) {
	log := testutils.GetLogger(t)
	ctx := log.WithContext(context.Background())
	db := database.GetTestDB(ctx, t, migration.Source)
	defer db.Close()

	// creating the user using the add handler
	addHandler := NewUserAddHandler(db.DB)

	user1, username1 := GetUser(t, "John", "Doe")
	user2, username2 := GetUser(t, "Jane", "Doe")
	_, err := addHandler.handle(ctx, user1)
	require.NoError(t, err)
	_, err = addHandler.handle(ctx, user2)
	require.NoError(t, err)

	// adding some dummy activity for the user JohnDoe
	qtyCurrentMonth1 := 4
	qtyLastMonth1 := 2
	renderList1 := AddUserActivity(t, username1, qtyCurrentMonth1, qtyLastMonth1)
	for _, rlog := range renderList1 {
		_, err = database.Exec(db, database.SQLInsert(rlog), &log)
		require.NoError(t, err)
	}
	qtyCurrentMonth2 := 4
	qtyLastMonth2 := 2
	renderList2 := AddUserActivity(t, username2, qtyCurrentMonth2, qtyLastMonth2)
	for _, rlog := range renderList2 {
		_, err = database.Exec(db, database.SQLInsert(rlog), &log)
		require.NoError(t, err)
	}

	t.Run("basic", func(t *testing.T) {
		getHandler := NewUserGetHandler(db.DB)
		user1, err := getHandler.handle(ctx, username1)
		require.NoError(t, err)
		stats := user1.Activity.RenderStats
		expectedTotal := qtyCurrentMonth1 + qtyLastMonth1
		require.Equal(t, int64(expectedTotal), stats.Total)
		require.Equal(t, int64(qtyCurrentMonth1), stats.CurrentMonth)
		require.Equal(t, int64(qtyLastMonth1), stats.LastMonth)

		user2, err := getHandler.handle(ctx, username2)
		require.NoError(t, err)
		stats = user2.Activity.RenderStats
		expectedTotal = qtyCurrentMonth2 + qtyLastMonth2
		require.Equal(t, int64(expectedTotal), stats.Total)
		require.Equal(t, int64(qtyCurrentMonth2), stats.CurrentMonth)
		require.Equal(t, int64(qtyLastMonth2), stats.LastMonth)
	})
}
