package tests

import (
	"context"
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestRegister(t *testing.T) {
	ctx := context.Background()
	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	t.Run("register", func(t *testing.T) {
		defer tester.SetT(t)

		if !assert.Equal(t, 204, tester.MustPostJSONCtx(ctx, "/api/v1/auth/register", apitester.JSONObj{
			"username":  "",
			"firstname": "Annie",
			"lastname":  "Ernaux",
			"email":     "example@example.com",
			"is-admin":  true,
		}, nil)) {
			t.FailNow()
		}

		db := database.NewSQLHelper(ctx, tester.DB, testutils.GetLogger(t))
		var userDB models.UserDB
		require.NoError(t, db.Get(
			&userDB, squirrel.Select("*").From("users").
				Where(squirrel.Eq{"email": "example@example.com"}),
		))

		assert.Equal(t, "example@example.com", userDB.Email)
		assert.Equal(t, "Annie", userDB.Firstname)
		assert.Equal(t, "Ernaux", userDB.Lastname)
		assert.Equal(t, false, userDB.IsAdmin)
	})
}
