package tests

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestUserManagement(t *testing.T) {
	ctx := context.Background()
	tester := apitester.NewAPITester(ctx, t, nil)
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	_ = t.Run("Create", func(t *testing.T) {
		defer tester.SetT(t)()
		var info models.Userinfo
		require.Equal(t, http.StatusCreated,
			tester.MustPostJSON("/api/v1/user", apitester.JSONObj{
				"username": "janedoe",
			}, &info))
		assert.Equal(t, "janedoe", info.Username)
		assert.False(t, info.IsAdmin)
	}) &&
		t.Run(
			"Get",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var info models.Userinfo
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/user/janedoe", &info),
				)
				assert.Equal(t, "janedoe", info.Username)
				assert.False(t, info.IsAdmin)
			}) &&
		t.Run(
			"List",
			func(t *testing.T) {
				defer tester.SetT(t)()
				var infoList []models.Userinfo
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/user", &infoList))
				assert.Equal(t, 2, len(infoList))
				assert.Equal(t, "admin", infoList[0].Username)
				assert.True(t, infoList[0].IsAdmin)
				assert.Equal(t, "janedoe", infoList[1].Username)
				assert.False(t, infoList[1].IsAdmin)
				assert.Contains(t, infoList, models.Userinfo{
					Username: "admin",
					IsAdmin:  true,
					Activity: &models.Activity{},
				})
				assert.Contains(t, infoList, models.Userinfo{
					Username:  "janedoe",
					IsAdmin:   false,
					CreatedAt: infoList[1].CreatedAt,
					Activity: &models.Activity{
						RenderStats: &models.RenderStats{
							Total:        int64(0),
							CurrentMonth: int64(0),
							LastMonth:    int64(0),
						},
					},
				})
			})
}
