package utils

import (
	"errors"
	"fmt"
	"strconv"
	"strings"
	"time"
)

type Period struct {
	From time.Time
	To   time.Time
}

func parseYearMonth(s string) (int, int, error) {
	if len(s) != 7 || s[4] != '-' {
		return 0, 0, errors.New("expects YYYY-MM")
	}
	year, err := strconv.Atoi(s[:4])
	if err != nil {
		return 0, 0, fmt.Errorf("error parsing the year: %w", err)
	}
	month, err := strconv.Atoi(s[5:])
	if err != nil {
		return 0, 0, fmt.Errorf("error parsing the month: %w", err)
	}

	return year, month, nil
}

func addMonth(year, month int) (int, int) {
	month++
	if month > 12 {
		month -= 12
		year++
	}

	return year, month
}

func ParsePeriod(s string) (*Period, error) {
	year, month, err := parseYearMonth(s[0:7])
	if err != nil {
		return nil, fmt.Errorf("invalid period: %w", err)
	}
	toYear, toMonth := year, month
	if len(s) > 7 && strings.Contains(s, "--") {
		if len(s) != 16 || s[4] != '-' || s[13] != '-' ||
			s[7] != '-' || s[8] != '-' {
			return nil, errors.New("invalid period. Expects YYYY-MM--YYYY-MM")
		}
		toYear, toMonth, err = parseYearMonth(s[9:16])
		if err != nil {
			return nil, fmt.Errorf("invalid period: %w", err)
		}
		if toYear < year {
			return nil, errors.New("invalid time period. date from must be before date to")
		}
		if toYear == year && toMonth < month {
			return nil, errors.New("invalid time period. date_from must be before date_to")
		}
	}
	toYear, toMonth = addMonth(toYear, toMonth)

	return &Period{
		time.Date(year, time.Month(month), 1, 0, 0, 0, 0, time.UTC),
		time.Date(toYear, time.Month(toMonth), 1, 0, 0, 0, 0, time.UTC),
	}, nil
}

func (p *Period) ToString() string {
	if p == nil {
		return "empty period"
	}

	return "From: " + p.From.Format(time.RFC3339) +
		", To: " + p.To.Format(time.RFC3339)
}
