package utils_test

import (
	"testing"

	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/utils"
)

func TestParsePeriod(t *testing.T) {
	for _, tt := range []struct {
		period         string
		expectedError  string
		expectedPeriod string
	}{
		{
			"2024-02",
			"",
			"From: 2024-02-01T00:00:00Z, To: 2024-03-01T00:00:00Z",
		},
		{
			"20244-02",
			"invalid period: expects YYYY-MM",
			"",
		},
		{
			"2024-02--2024-04",
			"",
			"From: 2024-02-01T00:00:00Z, To: 2024-05-01T00:00:00Z",
		},
		{
			"2024-03--2024-02",
			"invalid time period. date_from must be before date_to",
			"",
		},
	} {
		t.Run(tt.period, func(t *testing.T) {
			p, err := utils.ParsePeriod(tt.period)
			if err != nil {
				require.EqualError(t, err, tt.expectedError)
			}
			if p != nil {
				require.Equal(t, tt.expectedPeriod, p.ToString())
			}
		})
	}
}
